package vin.vio.collaboraonline.kingdee;


import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import jakarta.servlet.http.HttpServletRequest;
import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.entity.mime.HttpMultipartMode;
import org.apache.http.entity.mime.MultipartEntity;
import org.apache.http.entity.mime.content.InputStreamBody;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import vin.vio.collaboraonline.config.tenant.TenantConfigManager;
import vin.vio.collaboraonline.exception.AllKingdeeException;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @Description: TODO
 * @Date: 2025/2/21 16:25
 * @Created: by ZZSLL
 */

@Service
public class AuthService {
    @Autowired
    private TenantConfigManager manager;

    @Autowired
    private HttpServletRequest request;

    public String initBasePath() {

        Boolean isPublic = null;
        String testPublicEndPoint = null;
        String testInnerEndPoint = null;
        String prodPublicEndPoint = null;
        String prodInnerEndPoint = null;
        String env = null;

        String tenant = (String) request.getAttribute("tenant");
        Map<String, Object> config = manager.getConfigByTenant(tenant);

        if (config.containsKey("kingdee") && config.get("kingdee") instanceof Map<?,?>) {
            Map<String, Object> kingdee = (Map<String, Object>) config.get("kingdee");
            env = (String) kingdee.get("env");
            testPublicEndPoint = (String) kingdee.get("test-public-end-point");
            testInnerEndPoint = (String) kingdee.get("test-inner-end-point");
            prodPublicEndPoint = (String) kingdee.get("prod-public-end-point");
            prodInnerEndPoint = (String) kingdee.get("prod-inner-end-point");

            isPublic = (Boolean) kingdee.get("is-public");
        }

        if ("test".equals(env)) {
            if (isPublic) {
                return testPublicEndPoint;
            } else {
                return testInnerEndPoint;
            }
        }

        if ("prod".equals(env)) {
            if (isPublic) {
                return prodPublicEndPoint;
            } else {
                return prodInnerEndPoint;
            }
        }

        return "";
    }


    public String initAccessToken() throws IOException, AllKingdeeException {

        String tenant = (String) request.getAttribute("tenant");
        Map<String, Object> config = manager.getConfigByTenant(tenant);

        String appId = null;
        String appSecuret = null;
        String tenantid = null;
        String accountId = null;
        String language = null;

        if (config.containsKey("kingdee") && config.get("kingdee") instanceof Map<?,?>) {
            Map<String, Object> kingdee = (Map<String, Object>) config.get("kingdee");

            if (kingdee.containsKey("appToken") && kingdee.get("appToken") instanceof Map<?,?>) {
                Map<String, Object> appToken = (Map<String, Object>) kingdee.get("appToken");
                appId = (String) appToken.get("appId");
                appSecuret = (String) appToken.get("appSecuret");
                tenantid = (String) appToken.get("tenantid");
                accountId = (String) appToken.get("accountId");
                language = (String) appToken.get("language");
            }
        }

        final String END_POINT = initBasePath() + "/ierp/api/getAppToken.do";
        HttpClient client = new DefaultHttpClient();

        HttpPost post = new HttpPost(END_POINT);

        Map<String, String> body = new HashMap<>();

        body.put("appId", appId);
        body.put("appSecuret", appSecuret);
        body.put("tenantid", tenantid);
        body.put("accountId", accountId);
        body.put("language", language);

        HttpEntity reqEntity = new StringEntity(JSONUtil.toJsonStr(body), "utf-8");

        post.setEntity(reqEntity);
        post.setConfig(initRequestConfig());
        HttpResponse response = client.execute(post);
        if (response.getStatusLine().getStatusCode() == 200) {

            HttpEntity resEntity = response.getEntity();
            String responseData = EntityUtils.toString(resEntity, "utf-8");
            JSONObject jsonObject = JSON.parseObject(responseData);
            Boolean status = (Boolean) jsonObject.get("status");
            String state = (String) jsonObject.get("state");
            if (status && "success".equals(state)) {
                JSONObject data = (JSONObject) jsonObject.get("data");
                String appToken = (String) data.get("app_token");
                if (appToken != null && !appToken.isEmpty()) {
                    return doGetAccessToken(appToken);
                }
            }
        }
        throw new AllKingdeeException("获取AppToken失败");
    }

    private String doGetAccessToken(String appToken) throws IOException, AllKingdeeException {

        HttpClient client = new DefaultHttpClient();

        HttpPost post = new HttpPost(initBasePath() + "/ierp/api/login.do");

        Map<String, String> body = new HashMap<>();

        String user = null;
        String usertype = null;
        String tenantid = null;
        String accountId = null;
        String language = null;


        String tenant = (String) request.getAttribute("tenant");
        Map<String, Object> config = manager.getConfigByTenant(tenant);
        if (config.containsKey("kingdee") && config.get("kingdee") instanceof Map<?,?>) {
            Map<String, Object> kingdee = (Map<String, Object>) config.get("kingdee");

            if (kingdee.containsKey("accessToken") && kingdee.get("accessToken") instanceof Map<?,?>) {
                Map<String, Object> accessToken = (Map<String, Object>) kingdee.get("accessToken");
                user = (String) accessToken.get("user");
                usertype = (String) accessToken.get("usertype");
            }

            if (kingdee.containsKey("appToken") && kingdee.get("appToken") instanceof Map<?,?>) {
                Map<String, Object> appTokenParams = (Map<String, Object>) kingdee.get("appToken");
                tenantid = (String) appTokenParams.get("tenantid");
                accountId = (String) appTokenParams.get("accountId");
                language = (String) appTokenParams.get("language");
            }
        }

        body.put("user", user);
        body.put("apptoken", appToken);
        body.put("tenantid", tenantid);
        body.put("accountId", accountId);
        body.put("language", language);
        body.put("usertype", usertype);

        HttpEntity reqEntity = new StringEntity(JSONUtil.toJsonStr(body), "utf-8");

        post.setEntity(reqEntity);
        post.setConfig(initRequestConfig());
        HttpResponse response = client.execute(post);

        if (response.getStatusLine().getStatusCode() == 200) {

            HttpEntity resEntity = response.getEntity();
            String responseData = EntityUtils.toString(resEntity, "utf-8");
            JSONObject jsonObject = JSON.parseObject(responseData);
            Boolean status = (Boolean) jsonObject.get("status");
            String state = (String) jsonObject.get("state");

            if (status && "success".equals(state)) {
                JSONObject data = (JSONObject) jsonObject.get("data");
                String accessToken = (String) data.get("access_token");
                if (accessToken != null && !accessToken.isEmpty()) {
                    return accessToken;
                }
            }
        }
        throw new AllKingdeeException("获取AccessToken失败");
    }

    private RequestConfig initRequestConfig() {
        return RequestConfig.custom()
                .setConnectTimeout(5000)//一、连接超时：connectionTimeout-->指的是连接一个url的连接等待时间
                .setSocketTimeout(5000)// 二、读取数据超时：SocketTimeout-->指的是连接上一个url，获取response的返回等待时间
                .setConnectionRequestTimeout(5000)
                .build();
    }

}
