package vin.vio.collaboraonline.service;

import cn.hutool.log.Log;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpMethod;
import org.springframework.http.client.ClientHttpRequest;
import org.springframework.http.client.ClientHttpResponse;
import org.springframework.http.client.SimpleClientHttpRequestFactory;
import org.springframework.stereotype.Service;
import vin.vio.collaboraonline.config.tenant.TenantConfigManager;
import vin.vio.collaboraonline.exception.AllKingdeeException;
import vin.vio.collaboraonline.kingdee.AuthService;

import java.io.*;
import java.net.URI;
import java.net.URISyntaxException;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;

import static cn.hutool.http.HttpUtil.downloadFile;

/**
 * @Description: TODO
 * @Date: 2025/2/21 16:40
 * @Created: by ZZSLL
 */

@Service
public class KingdeeService {

    @Autowired
    private TenantConfigManager manager;

    @Autowired
    private AuthService authService;

    @Autowired
    private HttpServletRequest request;

    private final static Log log = Log.get();

    public String requestFile(String attPkId) {

        String BASE_DIR = null;
        String attachmentQueryURI = null;
        String fromId = null;
        String attachKey = null;

        String tenant = (String) request.getAttribute("tenant");
        Map<String, Object> config = manager.getConfigByTenant(tenant);
        if (config.containsKey("collabora") && config.get("collabora") instanceof Map<?,?>) {
            Map<String, Object> collabora = (Map<String, Object>) config.get("collabora");
            BASE_DIR = (String) collabora.get("file-storage-path");
        }

        if (config.containsKey("kingdee") && config.get("kingdee") instanceof Map<?,?>) {
            Map<String, Object> kingdee = (Map<String, Object>) config.get("kingdee");
            attachmentQueryURI = (String) kingdee.get("attachment-query-uri");

            if (kingdee.containsKey("bill") && kingdee.get("bill") instanceof Map<?,?>) {
                Map<String, Object> bill = (Map<String, Object>) kingdee.get("bill");
                fromId = (String) bill.get("formId");
                attachKey = (String) bill.get("attachKey");
            }
        }

        try {
            String accessToken = authService.initAccessToken();
            URI newUri = new URI(authService.initBasePath() + attachmentQueryURI);

            ClientHttpRequest delegate = new SimpleClientHttpRequestFactory().createRequest(newUri, HttpMethod.POST);
            delegate.getHeaders().add("access_token", accessToken);
            delegate.getHeaders().add("Content-Type", "application/json;charset=UTF-8");
            Map<String, String> requestBody = new HashMap<>();
            requestBody.put("formId", fromId);
            requestBody.put("pkId", attPkId);
            requestBody.put("attachKey", attachKey);
            String jsonBody = JSON.toJSONString(requestBody);
            // 设置请求体
            try (OutputStream os = delegate.getBody()) {
                os.write(jsonBody.getBytes(StandardCharsets.UTF_8));
            }
            ClientHttpResponse response = delegate.execute();
            BufferedReader reader = new BufferedReader(new InputStreamReader(response.getBody()));
            StringBuilder responseBody = new StringBuilder();
            String line;
            while ((line = reader.readLine()) != null) {
                responseBody.append(line);
            }
            String savePath = "";
            JSONObject result = JSON.parseObject(responseBody.toString());
            log.info(JSON.toJSONString(result));
            if (result.containsKey("status") && result.getBooleanValue("status", false)) {
                JSONArray data = result.getJSONArray("data");
                if (data != null && !data.isEmpty()) {
                    JSONObject data0 = data.getJSONObject(0);
                    String url = data0.getString("url");
                    String name = data0.getString("name");
                    savePath = BASE_DIR + "/" + name;
                    File existFile = new File(savePath);
                    if (existFile.exists()) {
                        existFile.delete();
                    }
                    downloadFile(url, savePath);
                }
            } else {
                throw new AllKingdeeException(line);
            }
            return savePath;
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException(e);
        }
    }
}
